/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package nl.cloudfarming.client.sensor;

import java.awt.Dialog;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import net.miginfocom.swing.MigLayout;
import nl.cloudfarming.client.geoviewer.LayerAction;
import nl.cloudfarming.client.geoviewer.Palette;
import nl.cloudfarming.client.model.SensorData;
import org.jdesktop.swingx.JXPanel;
import org.openide.DialogDescriptor;
import org.openide.DialogDisplayer;
import org.openide.util.Lookup;
import org.openide.util.NbBundle;
import org.openide.util.Utilities;

/**
 * Action to change the palette of the sensor data 
 * @author Merijn Zengers
 */
public final class ChangePaletteAction extends LayerAction{

    private Dialog dlg;
    private ArrayList options = new ArrayList();
    private JComboBox comboBox;
    private SensorDataObject sensorDataObject;
    private static final String DISPLAY_NAME = NbBundle.getMessage(ChangePaletteAction.class, "action.change_palette");
    private SensorFileLayer sensorFileLayer; 
    
    

    public ChangePaletteAction() {
        putValue(NAME, DISPLAY_NAME);
        
        sensorDataObject = Utilities.actionsGlobalContext().lookup(SensorDataObject.class);
        sensorFileLayer =  Utilities.actionsGlobalContext().lookup(SensorFileLayer.class);
        options = (ArrayList) getPaletteOptions();
        comboBox = new JComboBox(options.toArray());
        comboBox.setEditable(false); 
    }

    /**
     * Gets all the options for a pallete.
     * Will lookup all the Palette ServiceProviders
     * adds all the options of Palette to the returning ArrayList
     * @return the List with palette options
     */
    private List<SensorData> getPaletteOptions() {
        assert sensorDataObject != null;
        
        ArrayList paletteOptions = new ArrayList();
        Collection<? extends Palette> palettes = Lookup.getDefault().lookupAll(Palette.class);
        //The Object present in the lookup should be the selected object
        for (Palette palette : palettes) {
            if (palette.supports(sensorDataObject.getClass())) {
                if (palette.isInterchangeable()) {
                    paletteOptions.addAll(palette.getInterchangeablePalettes());
                } else {
                    paletteOptions.add(palette);
                }
            }
        }
        return paletteOptions;
    }

    @Override
    public void actionPerformed(ActionEvent arg0) {
        assert sensorDataObject != null;

        JXPanel panel = new JXPanel(new MigLayout("wrap 2"));

        comboBox.setSelectedItem(sensorDataObject.getPalette());
        panel.add(new JLabel("choose:"));
        panel.add(comboBox);

        DialogDescriptor dd = new DialogDescriptor(panel, DISPLAY_NAME);
        dd.setOptions(new Object[]{getOkButton(), getCancelButton()});
        dlg = DialogDisplayer.getDefault().createDialog(dd);
        dlg.setVisible(true);
    }

    private JButton getOkButton() {
        JButton jButtonOK = new JButton();
        jButtonOK.setText("Ok");
        jButtonOK.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent ae) {
                dlg.setVisible(false);
                if (comboBox.getSelectedItem() != null) {
                    sensorDataObject.setPalette((SensorPalette) comboBox.getSelectedItem());
                }

            }
        });
        return jButtonOK;
    }

    private JButton getCancelButton() {
        JButton jButtonCancel = new JButton();
        jButtonCancel.setText("cancel");
        jButtonCancel.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent ae) {
                dlg.setVisible(false);
            }
        });
        return jButtonCancel;
    }
}
